@@ -1,8 +1,9 @@
 //! Helpers for working with change sets.
 
-use std::{collections::HashSet, fmt, time::Duration};
+use std::{fmt, time::Duration};
 
 use chrono::{DateTime, Utc};
+use enumset::EnumSet;
 use futures_util::TryFutureExt;
 use memmem::{Searcher, TwoWaySearcher};
 use rusoto_cloudformation::{
@@ -76,6 +77,11 @@ pub struct ChangeSet {
     /// can’t execute the change set, the [`status`] indicates why. For example, a change set might
     /// be in an [`Unavailable`] state because AWS CloudFormation is still creating it or in an
     /// [`Obsolete`] state because the stack was already updated.
+    ///
+    /// [`Available`]: ExecutionStatus::Available
+    /// [`Obsolete`]: ExecutionStatus::Obsolete
+    /// [`Unavailable`]: ExecutionStatus::Unavailable
+    /// [`status`]: Self::status
     pub execution_status: ExecutionStatus,
 
     /// The Simple Notification Service (SNS) topic ARNs to publish stack related events.
@@ -194,10 +200,6 @@ pub struct ResourceChange {
     /// The action that AWS CloudFormation takes on the resource.
     pub action: Action,
 
-    /// For the [`Modify`](Action::Modify) action, a list of structures that describe the changes
-    /// that AWS CloudFormation will make to the resource.
-    pub details: Vec<ResourceChangeDetail>,
-
     /// The resource's logical ID, which is defined in the stack's template.
     pub logical_resource_id: String,
 
@@ -224,15 +226,10 @@ impl ResourceChange {
                     .action
                     .as_deref()
                     .expect("ResourceChange without action"),
+                change.details,
                 change.replacement,
                 change.scope,
             ),
-            details: change
-                .details
-                .unwrap_or_default()
-                .into_iter()
-                .map(ResourceChangeDetail::from_raw)
-                .collect(),
             logical_resource_id: change
                 .logical_resource_id
                 .expect("ResourceChange without logical_resource_id"),
@@ -251,14 +248,7 @@ pub enum Action {
     Add,
 
     /// Changes a resource.
-    Modify {
-        /// Indicates whether AWS CloudFormation will replace the resource by creating a new one and
-        /// deleting the old one.
-        replacement: Replacement,
-
-        /// Indicates which resource attribute is triggering this update.
-        scope: HashSet<ModifyScope>,
-    },
+    Modify(ModifyDetail),
 
     /// Deletes a resource.
     Remove,
@@ -271,9 +261,19 @@ pub enum Action {
 }
 
 impl Action {
-    fn from_raw(action: &str, replacement: Option<String>, scope: Option<Vec<String>>) -> Self {
+    fn from_raw(
+        action: &str,
+        details: Option<Vec<rusoto_cloudformation::ResourceChangeDetail>>,
+        replacement: Option<String>,
+        scope: Option<Vec<String>>,
+    ) -> Self {
         match action {
             "Add" | "Remove" | "Import" | "Dynamic" => {
+                assert!(
+                    matches!(details.as_deref(), None | Some([])),
+                    "ResourceChange with action {:?} and details",
+                    action
+                );
                 assert!(
                     replacement.is_none(),
                     "ResourceChange with action {:?} and replacement",
@@ -292,22 +292,55 @@ impl Action {
                     _ => unreachable!(),
                 }
             }
-            "Modify" => Self::Modify {
-                replacement: replacement
-                    .expect("ResourceChange with action \"Modify\" without replacement")
-                    .parse()
-                    .expect("ResourceChange with invalid replacement"),
-                scope: scope
-                    .expect("ResourceChange with action \"Modify\" without scope")
-                    .into_iter()
-                    .map(|scope| scope.parse().expect("ResourceChange with invalid scope"))
-                    .collect(),
-            },
+            "Modify" => Self::Modify(ModifyDetail::from_raw(
+                details.expect("ResourceChange with action \"Modify\" without details"),
+                &replacement.expect("ResourceChange with action \"Modify\" without replacement"),
+                &scope.expect("ResourceChange with action \"Modify\" without scope"),
+            )),
             _ => panic!("ResourceChange with invalid action {:?}", action),
         }
     }
 }
 
+/// Additional detail for resource modifications.
+#[derive(Clone, Debug, Eq, PartialEq)]
+pub struct ModifyDetail {
+    /// A list of structures that describe the changes that AWS CloudFormation will make to the
+    /// resource.
+    pub details: Vec<ResourceChangeDetail>,
+
+    /// Indicates whether AWS CloudFormation will replace the resource by creating a new one and
+    /// deleting the old one.
+    pub replacement: Replacement,
+
+    /// Indicates which resource attribute is triggering this update.
+    pub scope: EnumSet<ModifyScope>,
+}
+
+impl ModifyDetail {
+    fn from_raw(
+        details: Vec<rusoto_cloudformation::ResourceChangeDetail>,
+        replacement: &str,
+        scope: &[String],
+    ) -> Self {
+        Self {
+            details: details
+                .into_iter()
+                .map(ResourceChangeDetail::from_raw)
+                .collect(),
+            replacement: replacement
+                .parse()
+                .expect("ResourceChange with invalid replacement"),
+            scope: scope
+                .iter()
+                .map(|scope| -> ModifyScope {
+                    scope.parse().expect("ResourceChange with invalid scope")
+                })
+                .collect(),
+        }
+    }
+}
+
 /// Indicates whether AWS CloudFormation will replace the resource by creating a new one and
 /// deleting the old one.
 ///
@@ -320,6 +353,12 @@ impl Action {
 /// If you have multiple changes with different `requires_recreation` values, the `Replacement`
 /// value depends on the change with the most impact. A `requires_recreation` value of `Always` has
 /// the most impact, followed by [`Conditionally`], and then [`Never`].
+///
+/// [`Always`]: RequiresRecreation::Always
+/// [`Conditionally`]: RequiresRecreation::Conditionally
+/// [`Never`]: RequiresRecreation::Never
+/// [`Static`]: Evaluation::Static
+/// [`Dynamic`]: Evaluation::Dynamic
 #[derive(Clone, Copy, Debug, Eq, PartialEq, serde::Deserialize, serde::Serialize)]
 pub enum Replacement {
     /// The resource will be replaced.
@@ -336,7 +375,8 @@ forward_display_to_serde!(Replacement);
 forward_from_str_to_serde!(Replacement);
 
 /// Indicates which resource attribute is triggering this update.
-#[derive(Clone, Copy, Debug, Eq, Hash, PartialEq, serde::Deserialize, serde::Serialize)]
+#[derive(Debug, enumset::EnumSetType, serde::Deserialize, serde::Serialize)]
+#[enumset(no_ops, serialize_as_list)]
 pub enum ModifyScope {
     /// A change to the resource's properties.
     Properties,
